const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["./WorkflowList-DAeeH4j6.js","./api-DyiqpKJK.js","./api-BDer3rj7.css","./vue-core-C3hq3bxF.js","./element-ui--fm6Vjyy.js","./ui-components-DQ2lkQGO.js","./WorkflowList-Copuwi-a.css","./el-empty-D4ZqTl4F.css","./el-tag-DljBBxJR.css","./WorkflowStudio-BCcsM4bJ.js","./js-yaml-yTPt38rv.js","./elkjs-Cec2g0Wr.js","./WorkflowStudio-B8WFoV8O.css"])))=>i.map(i=>d[i]);
import{E as v,ag as w,y as g,ax as E,ay as P,an as L,az as O}from"./vue-core-C3hq3bxF.js";import{Q as k}from"./ui-components-DQ2lkQGO.js";(function(){const n=document.createElement("link").relList;if(n&&n.supports&&n.supports("modulepreload"))return;for(const e of document.querySelectorAll('link[rel="modulepreload"]'))l(e);new MutationObserver(e=>{for(const t of e)if(t.type==="childList")for(const r of t.addedNodes)r.tagName==="LINK"&&r.rel==="modulepreload"&&l(r)}).observe(document,{childList:!0,subtree:!0});function s(e){const t={};return e.integrity&&(t.integrity=e.integrity),e.referrerPolicy&&(t.referrerPolicy=e.referrerPolicy),e.crossOrigin==="use-credentials"?t.credentials="include":e.crossOrigin==="anonymous"?t.credentials="omit":t.credentials="same-origin",t}function l(e){if(e.ep)return;e.ep=!0;const t=s(e);fetch(e.href,t)}})();const b=(c,n)=>{const s=c.__vccOpts||c;for(const[l,e]of n)s[l]=e;return s},S={};function A(c,n){const s=w("router-view");return g(),v(s)}const N=b(S,[["render",A]]),R="modulepreload",$=function(c,n){return new URL(c,n).href},h={},_=function(n,s,l){let e=Promise.resolve();if(s&&s.length>0){let y=function(o){return Promise.all(o.map(u=>Promise.resolve(u).then(f=>({status:"fulfilled",value:f}),f=>({status:"rejected",reason:f}))))};const r=document.getElementsByTagName("link"),i=document.querySelector("meta[property=csp-nonce]"),p=i?.nonce||i?.getAttribute("nonce");e=y(s.map(o=>{if(o=$(o,l),o in h)return;h[o]=!0;const u=o.endsWith(".css"),f=u?'[rel="stylesheet"]':"";if(l)for(let d=r.length-1;d>=0;d--){const m=r[d];if(m.href===o&&(!u||m.rel==="stylesheet"))return}else if(document.querySelector(`link[href="${o}"]${f}`))return;const a=document.createElement("link");if(a.rel=u?"stylesheet":R,u||(a.as="script"),a.crossOrigin="",a.href=o,p&&a.setAttribute("nonce",p),document.head.appendChild(a),u)return new Promise((d,m)=>{a.addEventListener("load",d),a.addEventListener("error",()=>m(new Error(`Unable to preload CSS for ${o}`)))})}))}function t(r){const i=new Event("vite:preloadError",{cancelable:!0});if(i.payload=r,window.dispatchEvent(i),!i.defaultPrevented)throw r}return e.then(r=>{for(const i of r||[])i.status==="rejected"&&t(i.reason);return n().catch(t)})},x=()=>_(()=>import("./WorkflowList-DAeeH4j6.js"),__vite__mapDeps([0,1,2,3,4,5,6,7,8]),import.meta.url),C=()=>_(()=>import("./WorkflowStudio-BCcsM4bJ.js").then(c=>c.j),__vite__mapDeps([9,1,2,3,5,4,10,11,12]),import.meta.url),W=[{path:"/",name:"workflow-list",component:x},{path:"/workflow/:id",name:"workflow-studio",component:C}],j=E({history:P(),routes:W}),q=L(N).use(j).use(k).use(O());q.mount("#app");export{b as _,_ as a};
