import"./api-DyiqpKJK.js";/* empty css               */import"./WorkflowEditor-BYbpWGbG.js";import"./WorkflowStudio-BCcsM4bJ.js";import{J as s}from"./element-ui--fm6Vjyy.js";import{v as p,r as c,x as u,y as l,O as f}from"./vue-core-C3hq3bxF.js";import{_ as m}from"./index-B2BzKGz2.js";const g={year:"numeric",month:"short",day:"numeric",hour:"2-digit",minute:"2-digit"};function D(e,o){if(!e)return"";const a=new Date(e),r=new Date,t=a.getTime()-r.getTime();if(!o?.absolute){if(t<6e4)return"刚刚";if(t<36e5)return`${Math.floor(t/6e4)}分钟前`;if(t<864e5)return`${Math.floor(t/36e5)}小时前`;if(t<6048e5)return`${Math.floor(t/864e5)}天前`}return a.toLocaleDateString("zh-CN",{...g,...o?.format})}const d={class:"pagination-container"},_=p({__name:"FloatPagination",props:{total:{},pageSize:{},reload:{type:Function}},setup(e){const o=e,a=c(1);return(r,t)=>{const i=s;return l(),u("div",d,[f(i,{layout:"prev, pager, next",total:o.total,"page-size":o.pageSize,"pager-count":5,"current-page":a.value,"onUpdate:currentPage":t[0]||(t[0]=n=>a.value=n),onChange:t[1]||(t[1]=n=>e.reload(n))},null,8,["total","page-size","current-page"])])}}}),P=m(_,[["__scopeId","data-v-f82999d7"]]);export{P as F,D as f};
